/* $Id: 7c_def.h,v 1.1 2004/07/29 14:17:24 berar Exp $ */
/*   7c_def.h   header des fonctions 7c */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* ------------------------------------------------------*/
typedef union
{     double v[3];
      struct 
      {  double  x;
         double  y;
         double  z;
         } coord; 
		}  vector_type;

typedef union
{     double t[9];
      double m[3][3];
      vector_type v[3];      
		}  matrix_type;
      
typedef struct
{    double a, b, c, alpha, beta, gamma;
	  } cell_type;
     
     
typedef enum { OK, INVALID, Q_RANGE, DET_RANGE, MODE_RANGE } valid_type;

#define GEOM 9
/*  crystal angles are rho (Z), omega (Y), chi (X), phi (Y) */
/* detector angles are mu (Z) and nu (Y) */
typedef union
{   struct
    {   double rho, omega, chi, phi;
        double mu, nu;   
        double rx, ry, rz;
        }  pos;
      double angle[GEOM];      
		}  geom_type;

typedef union
{   struct
    {   double bragg, theta;   
        double alpha, beta, azimuth ;  
        }  pos;
      double angle[5];      
      }  bragg_type;  

typedef struct
{    valid_type  ok;
     double dist;
	  vector_type hkl;         /* miller index */
     geom_type   result;
     bragg_type  bragg;
     } hkl_type;
        
/* ------------------------------------------------------*/
/*  variables globales , EXTERN is defined in 7c_cry.c */      
#ifndef EXTERN 
#define EXTERN extern
#endif
EXTERN char * date_7c_cry;
EXTERN char * time_7c_cry;

EXTERN int p_errormsg;      /* not zero to print error msg */
EXTERN matrix_type mat_ub;
		  /* components of reciprocal axis on the gonio head */
EXTERN matrix_type mat_inv_ub;        /* inversed ub matrix*/
EXTERN matrix_type mat_u;
		/* rotation matrix of ortho recip base to gonio head */
EXTERN matrix_type mat_b;
		  /* components of reciprocal axis on its ortho base */
EXTERN vector_type u_angles;    /*  rx, ry, rz values of mat_u */

EXTERN double  lambda;
EXTERN vector_type  hkl_cryst;    /* miller index */

EXTERN vector_type  azimuth;
EXTERN double  azimuth_angle;
EXTERN geom_type goniometer; /* the goniometer setting */

EXTERN hkl_type  hkl;       /* current hkl */
EXTERN hkl_type  or0;       /* orien vec 0 */
EXTERN hkl_type  or1;       /* orien vec 1 */

EXTERN cell_type    direct;
EXTERN cell_type    recip;

EXTERN int     spec_ub;      /* set to 1 to use 2Pi in recip space */

#define MAX_MODE   5         /* at the time, used only in printing */

EXTERN int     spec_mode;    /* used like in spec to specify calculation */
/* ------------------------------------------------------*/
#ifndef M_PI
#define M_PI 3.14159
#endif
#ifndef M_2PI
#define M_2PI (2.0*M_PI)
#endif
#define TO_DEG  (180.0/M_PI)
/* ------------------------------------------------------*/
/* public functions known in 7c_cry.c  */
char * mode_msg( int mode );

void cell_from_ub(void);

void get_hkl(void);
void get_angles(void);    
void ub_from_or(void);

/* ------------------------------------------------------*/
/* ------------------------------------------------------*/
/* public functions known in 7c_use.c  */
extern char * date_7c_use;
extern char * time_7c_use;

int solve_trigo( double * sol, double a, double b, double c);
int solve_x2( double * sol, double a, double b, double c);

int solve_rot( int axe, double * sol, vector_type *u, vector_type *v);

int     inv_mat(matrix_type * res, matrix_type * mat);
void mul_matvec(vector_type  * res, matrix_type * mat, vector_type * vec );
void mul_matmat(matrix_type  * res, matrix_type * mat, matrix_type * vec );
void rotate ( int axe, vector_type *r, double angle, vector_type *s);

void put_vector( char * id, vector_type * v);
void put_matrix( char * id, matrix_type * m);
/* ------------------------------------------------------*/
